/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.lang.reflect.Method;

import attila.core.auth.*;

/**
 * Encapsulates a single action that is open to the web.
 * @author Kai Kousa
 * */
@SuppressWarnings("unchecked")
public class AttilaAction {
	
	private Method action;
	private boolean authenticated = false;
	private HttpMethodEnum httpMethod = HttpMethodEnum.GET;
	private String actionSignature = "";
	
	private AuthenticationLevel authenticationLevel = new DefaultAuthenticationLevel();
	private Class authenticationLevelErrorController = null;
	private String authenticationLevelErrorControllerAction = "";
	
	public AttilaAction(Method action, boolean authenticated){
		this.action = action;
		this.authenticated = authenticated;
		generateActionSignature();
	}
	
	public AttilaAction(Method action, boolean authenticated, HttpMethodEnum httpMethod){
		this.action = action;
		this.authenticated = authenticated;
		this.httpMethod = httpMethod;
		generateActionSignature();
	}
	
	/**
	 * Returns a Method in a controller.
	 * @return <code>Method</code> 
	 * */
	public Method getAction(){
		return action;
	}
	
	/**
	 * Returns the name of this action (precisely the name of the method that performs the action).
	 * */
	public String getName(){
		return action.getName();
	}
	
	/**
	 * If true, Attila must check that the user trying to access this action is 
	 * authenticated and logged in. If false, no such validation need to occur.
	 * */
	public boolean isAuthenticated(){
		return authenticated;
	}
	
	public HttpMethodEnum getHttpMethod(){
		return httpMethod;
	}

	/**
	 * @return <code>AuthenticationLevel</code> of this action. 
	 * Default implementation is <code>DefaultAuthenticationLevel</code>
	 * */
	public AuthenticationLevel getAuthenticationLevel() {
		return authenticationLevel;
	}

	/**
	 * Set <code>AuthenticationLevel</code> for this action.
	 * @param authenticationLevel - 
	 * */
	public void setAuthenticationLevel(AuthenticationLevel authenticationLevel) {
		this.authenticationLevel = authenticationLevel;
	}

	public Class getAuthenticationLevelErrorController() {
		return authenticationLevelErrorController;
	}

	/**
	 * Set the controller to which the user will be directed if her authentication leve validation fails
	 * */
	public void setAuthenticationLevelErrorController(Class authenticationLevelErrorController) {
		this.authenticationLevelErrorController = authenticationLevelErrorController;
	}

	public String getAuthenticationLevelErrorControllerAction() {
		return authenticationLevelErrorControllerAction;
	}

	/**
	 * Set the action of a controller to which the user will be directed if her authentication level validation fails
	 * */
	public void setAuthenticationLevelErrorControllerAction(String authenticationLevelErrorControllerAction) {
		this.authenticationLevelErrorControllerAction = authenticationLevelErrorControllerAction;
	}
	
	/**
	 * Has controller been defined for redirect?
	 * */
	public boolean hasAuthenticationLevelErrorController(){
		if(authenticationLevelErrorController != null){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Has action been defined for redirect?
	 * */
	public boolean hasAuthenticationLevelErrorControllerAction(){
		if(authenticationLevelErrorControllerAction != null && authenticationLevelErrorControllerAction.length() > 0){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Returns this action's signature ([action name]:[number of parameters])
	 * */
	public String getActionSignature(){
		return actionSignature;
	}
	
	/**
	 * Generates a signature for this action ([action name]:[number of parameters]
	 * */
	private void generateActionSignature(){
		actionSignature = action.getName() + ":";
		actionSignature += action.getParameterTypes().length - 2; //request and response parameters are not counted in
	}
	
}
