/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.io.IOException;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import attila.core.util.*;

/**
 * Adds some useful methods for actual controllers.
 * @author Kai Kousa
 * */
public abstract class AttilaTheController extends AttilaTheServlet{
	
	/**
	 * Redirects the user to an absolute location.
	 * @param redirectUrl - eg. http://www.example.com
	 * @param response - instance of <code>HttpServletResponse</code>
	 * */
	protected void redirect(String redirectUrl, HttpServletResponse response) throws IOException{
		response.sendRedirect(redirectUrl);
	}
	
	
	/**
	 * Redirects the browser to an exact servlet regardless of the server name or the configured url of 
	 * the servlet.
	 * @param servletClass - Class of the Servlet.
	 * @param response - instance of <code>HttpServletResponse</code>
	 * */
	@SuppressWarnings("unchecked")
	protected void redirectToServlet(Class servletClass, HttpServletResponse response) throws IOException{
		response.sendRedirect(ServletUrlUtil.getServletUrl(servletClass.getName(), getServletContext()));
	}
	
	@SuppressWarnings("unchecked")
	protected void redirectToServlet(Class servletClass, HttpServletResponse response, Map<String, String> params) throws IOException{
		StringBuilder paramStr = new StringBuilder();
		
		for(String key : params.keySet()){
			String value = params.get(key);
			paramStr.append(key);
			paramStr.append("=");
			paramStr.append(java.net.URLEncoder.encode(value, "utf8"));
			paramStr.append("&");
		}
		response.sendRedirect(ServletUrlUtil.getServletUrl(servletClass.getName(), getServletContext()) + "?" + paramStr.toString());
	}
	
	/**
	 * Redirects to an action. Be careful as this generates restful urls only (for now).
	 * @param servletClass - the <code>Class</code> of the receiving controller
	 * @param action - name of the action in the receiving controller
	 * @param response - the current response
	 * @param params - parameters for the action
	 * */
	@SuppressWarnings("unchecked")
	protected void redirectToAction(Class servletClass, String action, HttpServletResponse response, Object... params) throws IOException{
		String paramStr = "";
		for(Object param : params){
			paramStr += "/" + param.toString();
		}
		response.sendRedirect(ServletUrlUtil.getServletUrl(servletClass.getName(), getServletContext()) + "/" + action + paramStr);
	}
	
	/**
	 * Dispatches the control to a named servlet.
	 * @param servletName - the name that is assigned to the servlet in web.xml
	 * @param request - instance of <code>HttpServletRequest</code>
	 * @param response - instance of <code>HttpServletResponse</code>
	 * */
	@SuppressWarnings("unchecked")
	protected void dispatchToServlet(Class servletClass, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		Map<String, String> servletMappings = ServletUrlUtil.getServletMappings(getServletContext());
		
		String servletPath = servletMappings.get(servletClass.getName());		
		if(!servletPath.startsWith("/")){
			servletPath = "/" + servletPath;
		}
		
		getServletContext().getRequestDispatcher(servletPath).forward(request, response);
	}
	
	/**
	 * Sends the browser back to where it came from.
	 * @param request - instance of <code>HttpServletRequest</code>
	 * @param response - instance of <code>HttpServletResponse</code>
	 * */
	protected void redirectBack(HttpServletRequest request, HttpServletResponse response) throws IOException{
		String referer = request.getHeader("Referer");
		if(referer != null){
			response.sendRedirect(referer);			
		}else{
			response.sendRedirect(ServletUrlUtil.getContextPath(getServletContext()));
		}
	}
	
}
