/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.io.IOException;
import java.util.*;

import javax.servlet.*;
import javax.servlet.http.*;

import attila.core.auth.*;
import attila.core.exception.*;
import attila.core.util.*;
import attila.core.event.*;

/**
 * The very heart of Attila's controllers. Each class that inherits <code>AttilaTheServlet</code>
 * gains the ability to easily expose methods to the web through simple annotations.
 * @author Kai Kousa
 * */
public abstract class AttilaTheServlet extends HttpServlet {

	//Lookup map for open-to-web methods
	private AttilaActionLookup getMethodLookup = new AttilaActionLookup();
	private AttilaActionLookup postMethodLookup = new AttilaActionLookup();
	private AttilaActionLookup deleteMethodLookup = new AttilaActionLookup();
	private AttilaActionLookup putMethodLookup = new AttilaActionLookup();
	
	private ActionResolver actionResolver;
	private ActionInvoker actionInvoker;
	
	private List<ActionListener> actionListeners = new ArrayList<ActionListener>();
	
	private String authenticationControllerClassName = null;
	
	@Override
	public void init(ServletConfig config) throws ServletException {
		super.init(config);
		
		actionResolver = new RESTActionResolver();
		actionInvoker = actionResolver.getActionInvoker();
		
		//Parse public web-methods from the controller and store them in different lookups for later use
		getMethodLookup = ActionParser.createActionLookup(getClass(), HttpMethodEnum.GET);
		postMethodLookup = ActionParser.createActionLookup(getClass(), HttpMethodEnum.POST);
		deleteMethodLookup = ActionParser.createActionLookup(getClass(), HttpMethodEnum.DELETE);
		putMethodLookup = ActionParser.createActionLookup(getClass(), HttpMethodEnum.PUT);
		
		//As index method is a bit special compared to others, we'll add it implicitly if it hasn't been explicitly added to 
		//postMethodLookup by an annotation.
		if(!postMethodLookup.hasIndex()){
			getMethodLookup.addAction(ActionParser.createAction(getClass(), "index"));
		}
		
		authenticationControllerClassName = getServletContext().getInitParameter("attila.authenticationController");
	}
	
	@Override
	protected void doPost(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try{
			AttilaRequest wrappedRequest = new AttilaRequest(request);
			AttilaAction action = actionResolver.resolveAction(wrappedRequest, postMethodLookup);
			processActionRequest(action, wrappedRequest, response);	
		}catch(Exception ex){
			handleException(ex, request, response);
		}
	}
	
	@Override
	protected void doGet(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try{
			AttilaRequest wrappedRequest = new AttilaRequest(request);
			AttilaAction action = actionResolver.resolveAction(wrappedRequest, getMethodLookup);
			processActionRequest(action, wrappedRequest, response);
		}catch(Exception ex){
			handleException(ex, request, response);
		}
	}
	
	@Override
	protected void doDelete(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			AttilaRequest wrappedRequest = new AttilaRequest(request);
			AttilaAction action = actionResolver.resolveAction(wrappedRequest, deleteMethodLookup);
			processActionRequest(action, wrappedRequest, response);
		} catch (Exception e) {
			handleException(e, request, response);
		}
	}
	
	@Override
	protected void doPut(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
		try {
			AttilaRequest wrappedRequest = new AttilaRequest(request);
			AttilaAction action = actionResolver.resolveAction(wrappedRequest, putMethodLookup);
			processActionRequest(action, wrappedRequest, response);
		} catch (Exception e) {
			handleException(e, request, response);
		}
	}
	
	/**
	 * Default action of each controller. index is by default accessible through GET-requests, but can be configured for POST with @HttpMethod(method=HttpMethodEnum.POST)-annotation.
	 * */
	public abstract void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException;
	
	/**
	 * Add an ActionListener to this AttilaTheServlet-instance
	 * */
	public void addActionListener(ActionListener listener){
		actionListeners.add(listener);
	}
	
	private void launchPreActionInvokeEvents(ActionInvokeEvent event){
		for(ActionListener listener : actionListeners){
			listener.preActionInvoke(event);
		}
	}
	
	private void launchPostActionInvokeEvents(ActionInvokeEvent event){
		for(ActionListener listener : actionListeners){
			listener.postActionInvoke(event);
		}
	}
	
	@SuppressWarnings("unchecked")
	private void handleException(Exception ex, HttpServletRequest request, HttpServletResponse response) {
		String exceptionHandlerClassName = getServletContext().getInitParameter("attila.defaultExceptionHandler");
		if(exceptionHandlerClassName != null){
			try {
				Class exceptionHandlerClass = Class.forName(exceptionHandlerClassName);
				ExceptionHandler exceptionHandler = (ExceptionHandler) exceptionHandlerClass.newInstance();
				
				exceptionHandler.handleException(ex, request, response);
			} catch (ClassNotFoundException e) {
				e.printStackTrace(); 
				new DefaultExceptionHandler().handleException(ex, request, response);
			} catch (InstantiationException e) {
				e.printStackTrace();
				new DefaultExceptionHandler().handleException(ex, request, response);
			} catch (IllegalAccessException e) {
				e.printStackTrace();
				new DefaultExceptionHandler().handleException(ex, request, response);
			}
		}else{
			new DefaultExceptionHandler().handleException(ex, request, response);			
		}
		
	}
	
	/**
	 * Checks the authentication configurations and invokes the given action.
	 * */
	private void processActionRequest(AttilaAction action, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException, Exception{
		if(action.isAuthenticated()){
			UserSession<AttilaTheUser> userSession = new UserSession<AttilaTheUser>(request, response);
			if(userSession.isAuthenticated()){
				
				if(action.getAuthenticationLevel().checkUserAuthenticationLevel(userSession.getUser())){
					invokeAttilaAction(action, request, response);
				}else{
					//if configured, redirect to another controller
					if(action.hasAuthenticationLevelErrorController()){
						String errorControllerUrl = ServletUrlUtil.getServletUrl(action.getAuthenticationLevelErrorController().getName(), getServletContext());
						
						//if configured, redirect to the specified action
						if(action.hasAuthenticationLevelErrorControllerAction()){
							errorControllerUrl += "/" + action.getAuthenticationLevelErrorControllerAction();
						}
						
						response.sendRedirect(errorControllerUrl);
					}else{
						throw new AuthenticationLevelValidationException("The user does not have correct usage rights for this resource.");
					}
				}
				
			}else{
				redirectToLogin(request, response);
			}
			
		}else{//no need to check authentication, process as usual
			invokeAttilaAction(action, request, response);
		}
	}
	
	/**
	 * Invokes the given action.
	 * */
	private void invokeAttilaAction(AttilaAction action, HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException, Exception{
		launchPreActionInvokeEvents(new ActionInvokeEvent(action, request, response));
		actionInvoker.invokeAction(action, this, request, response);
		launchPostActionInvokeEvents(new ActionInvokeEvent(action, request, response));
	}
	
	/**
	 * Redirect the user to login page. Fail miserably if the programmer has not configured the authentication controller.
	 * */
	private void redirectToLogin(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException{
		if(authenticationControllerClassName != null){
			String authenticationUrl = ServletUrlUtil.getServletUrl(authenticationControllerClassName, getServletContext());
			if(authenticationUrl != null){
				response.sendRedirect(authenticationUrl);
			}else{
				throw new FrameworkMisconfiguredException("Servlet for the authentication controller not found. Check your web.xml");
			}
		}else{
			throw new FrameworkMisconfiguredException("You must specify and configure an authentication controller if you wish to use Authentication-annotations. Check your web.xml.");
		}
	}
	
}
