/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.util.*;
import java.lang.reflect.*;

import javax.servlet.http.*;

import attila.core.event.*;

/**
 * Abstract superclass for Attila's view-components.
 * @author Kai Kousa
 * */
public abstract class AttilaTheViewable {

	private List<ViewListener> viewListeners = new ArrayList<ViewListener>();
	
	/**
	 * Add a <code>ViewListener</code> to this viewable.
	 * */
	public void addViewListener(ViewListener listener){
		viewListeners.add(listener);
	}
	
	public List<ViewListener> getViewListeners(){
		return viewListeners;
	}
	
	/**
	 * Gets data from extending class's public getters and pushes that data 
	 * to a <code>Map</code> with a key that is generated from the getter's method name.
	 * Eg. public User getUser => data.put("user", getUser());
	 * @return <code>Map</code> that 
	 * */
	public Map<String, Object> prepareDataForRendering(){
		Map<String, Object> data = new HashMap<String, Object>();
		
		Method[] methods = this.getClass().getMethods();
		for(Method m : methods){
			if(m.getModifiers() == Modifier.PUBLIC){
				String methodName = m.getName();
				if(methodName.startsWith("get")){
					String key = createPropertyName(methodName, "get");
					Object value = getValueFromSource(this, m);
					data.put(key, value);
				}else if(methodName.startsWith("is")){
					String key = createPropertyName(methodName, "is");
					Object value = getValueFromSource(this, m);
					data.put(key, value);
				}
			}
		}
		
		return data;
	}
	
	/**
	 * Dumps the prepared data to the current request and calls subclass's renderView-method
	 * for actual rendering. Launches preRender- and postRender-events.
	 * */
	public void render(HttpServletRequest request, HttpServletResponse response){
		launchPreRenderEvents(request, response);
		
		Map<String, Object> preparedData = prepareDataForRendering();
		
		for(String key : preparedData.keySet()){
			Object bean = preparedData.get(key);
			request.setAttribute(key, bean);
		}
		
		renderView(request, response);
		
		launchPostRenderEvents(request, response);
	}
	
	/**
	 * The actual rendering method. Subclasses choose how the view gets rendered.
	 * */
	protected abstract void renderView(HttpServletRequest request, HttpServletResponse response);
	
	private void launchPreRenderEvents(HttpServletRequest request, HttpServletResponse response){
		for(ViewListener listener : viewListeners){
			listener.preRender(new ViewRenderEvent(this, request, response));
		}
	}
	
	private void launchPostRenderEvents(HttpServletRequest request, HttpServletResponse response){
		for(ViewListener listener : viewListeners){
			listener.postRender(new ViewRenderEvent(this, request, response));
		}
	}
	
	/**
	 * Calls the specified method on the source and captures the result. Also
	 * handles a bunch of exceptions and returns null if an exception is encountered
	 * @param source - the source object
	 * @param m - method which is invoked on the source
	 * */
	private Object getValueFromSource(Object source, Method m){
		Object result = null;
		try {
			Object[] params = null;
			result = m.invoke(source, params);
		} catch (IllegalArgumentException e) {
			e.printStackTrace();
		} catch (IllegalAccessException e) {
			e.printStackTrace();
		} catch (InvocationTargetException e) {
			e.printStackTrace();
		}
		return result;
	}
	
	/**
	 * Takes a JavaBean-style method name and modifies it to a property name
	 * Example: 'setName' => 'name', 'setUserLastName' => 'userLastName'.
	 * @param methodName - name of the method
	 * @param prefix - prefix of the method (is, set...)
	 * */
	protected String createPropertyName(String methodName, String prefix){
		String propertyName = methodName.replaceFirst(prefix, "");		
		String result = Character.toLowerCase(propertyName.charAt(0)) + propertyName.substring(1);
		
		return result;
	}
	
}
