/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.util.*;

import javax.servlet.http.*;

import attila.core.util.*;
import attila.core.exception.*;

/**
 * Invokes REST'ish actions. Checks action method's parameters and automatically casts parameters from the url to the 
 * type that is declared in the method. Currently supports int, float, long, boolean and String parameters.
 * @author Kai Kousa
 * */
public class RESTActionInvoker implements ActionInvoker{

	@Override
	public void invokeAction(AttilaAction action, AttilaTheServlet servlet, HttpServletRequest request, HttpServletResponse response) throws Exception {
		RESTUrl restUrl = new RESTUrl(request);
		
		List<String> parameterValues = restUrl.getParameterValues();
		
		Class<?>[] paramTypes = action.getAction().getParameterTypes();			
		Object[] params = new Object[paramTypes.length];
		
		if(parameterValues.size() != (paramTypes.length) - 2){//take out HttpServletRequest and HttpServletResponse parameters as they're not parameters that are passed from the url
			throw new ActionInvocationException("Wrong number of arguments. Check the action's parameters in the controller");
		}
		
		for(int i = 0; i < paramTypes.length; i++){
			Class<?> paramType = paramTypes[i];
			
			String paramTypeName = paramType.getName();
			if(paramTypeName.equals(MultipartRequest.class.getName())){
				params[i] = new MultipartRequest(request);
			}else if(paramTypeName.equals(HttpServletRequest.class.getName())){
				params[i] = request;
			}else if(paramTypeName.equals(HttpServletResponse.class.getName())){
				params[i] = response;
			}else if(paramTypeName.equals("int") || paramTypeName.equals("java.lang.Integer")){		
				params[i] = Integer.parseInt(parameterValues.get(i));
			}else if(paramTypeName.equals("java.lang.String")){
				params[i] = parameterValues.get(i);
			}else if(paramTypeName.equals("float") || paramTypeName.equals("java.lang.Float")){
				params[i] = Float.parseFloat(parameterValues.get(i));
			}else if(paramTypeName.equals("long") || paramTypeName.equals("java.lang.Long")){
				params[i] = Long.parseLong(parameterValues.get(i));
			}else if(paramTypeName.equals("boolean")  || paramTypeName.equals("java.lang.Boolean")){
				String booleanParam = parameterValues.get(i);
				if(booleanParam.equals("true") || booleanParam.equals("1")){
					params[i] = new Boolean(true);
				}else if(booleanParam.equals("false") || booleanParam.equals("0")){
					params[i] = new Boolean(false);
				}else{
					throw new ActionInvocationException("Boolean parameters should only be passed as true, false or 1, 0.");
				}
			}
		}
		
		action.getAction().invoke(servlet, params);

	}

}
