/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core.auth;

import javax.servlet.http.*;

/**
 * Simplifies the checking of authenticated users.
 * @author Kai Koousa
 * */
public class UserSession <T extends AttilaTheUser> {
	
	public static final String USER_DATA_SESSION_KEY = "attila.userData";
	
	private HttpServletRequest request = null;
	private HttpServletResponse response = null;
	private T user = null;
	private boolean authenticated = false;
	private boolean authenticatedByPassword = false;
	
	/**
	 * Default constructor
	 * */
	public UserSession() {}
	
	/**
	 * Creates an instance of <code>UserSession</code>
	 * @param request - instance of <code>HttpServletRequest</code>
	 * */
	@SuppressWarnings("unchecked")
	public UserSession(HttpServletRequest request, HttpServletResponse response){
		this.request = request;
		this.response = response;
		
		Object userObject = this.request.getSession().getAttribute(USER_DATA_SESSION_KEY);
		if(userObject != null){
			user = (T) userObject;
		}
		authenticated = checkAuthentication();
	}
	
	@SuppressWarnings("unchecked")
	public UserSession(HttpSession session){
		Object userObject = session.getAttribute(USER_DATA_SESSION_KEY);
		if(userObject != null){
			user = (T) userObject;
		}
		authenticated = checkAuthentication();
	}
	
	/**
	 * Returns true if the user is authenticated. True is returned 
	 * regardless of the method of authentication(form, cookie).
	 * @return the authentication status of the user
	 * */
	public boolean isAuthenticated(){
		return authenticated;
	}
	
	/**
	 * Useful for checking if the user logged in using password (login-form) or cookie (automatic login)
	 * @return the authenticationByPassword status of the user
	 * */
	public boolean isAuthenticatedByPasword(){
		return authenticatedByPassword;
	}
	
	/**
	 * Returns the User-object. This method should only be called if isAuthenticated returns true (returns null otherwise).
	 * */
	public T getUser(){
		return user;
	}
	
	/**
	 * Save user's current session
	 * */
	public void save(){
		setUserData(request, user);
	}
	
	/**
	 * Destroy user's session
	 * */
	public void delete(){
		setUserData(request, null);
	}
	
	/**
	 * Push the user data to the session. Uses a pre-defined key for storing the data in the session.
	 * */
	public static void setUserData(HttpServletRequest request, Object userData){
		request.getSession().setAttribute(USER_DATA_SESSION_KEY, userData);
	}
	
	/**
	 * Check the authentication of the user
	 * */
	@SuppressWarnings("unchecked")
	private boolean checkAuthentication(){

		if(user != null){
			return user.isAuthenticated();
		}else{
			CookieAuthenticator cookieAuth = CookieAuthenticatorFactory.createCookieAuthenticator();
			if(cookieAuth != null){
				if(cookieAuth.checkAuthentication(request, response)){
					user = (T)cookieAuth.getUser();
					user.setAuthenticated(true);
					user.setAuthenticatedByPassword(false);
					save();
					
					return true;
				}else{
					return false;
				}
			}else{
				return false;
			}
		}
		
	}
}
