/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core.util;

import java.lang.reflect.*;
import java.util.*;

import attila.core.*;
import attila.core.auth.*;

/**
 * Utility class for parsing <code>AttilaActions</code> from a class that has methods annotated 
 * with <code>HTTPMethod</code>-annotation.
 * @author Kai Kousa
 * */
public class ActionParser {
	
	/**
	 * Parse <code>AttilaActions</code> from a given class.
	 * @param containerClass - a <code>Class</code> that has <code>HTTPMethod</code> annotated public methods
	 * @param method - the method type that should be parsed (eg. GET or POST)
	 * */
	@SuppressWarnings("unchecked")
	public static AttilaActionLookup createActionLookup(Class containerClass, HttpMethodEnum method){
		Map<String, AttilaAction> actions = new HashMap<String, AttilaAction>();
		Map<String, AttilaAction> actionsBySignature = new HashMap<String, AttilaAction>();
		
		//Extract the methods for faster lookup
		Method[] methods = containerClass.getMethods();
		for(Method m : methods){
			if(m.getModifiers() == Modifier.PUBLIC){
				AttilaAction action = createAction(containerClass, m);
				
				HTTPMethod httpMethod = m.getAnnotation(HTTPMethod.class);
				if(httpMethod != null){
					if(httpMethod.method() == method){
						actions.put(m.getName(), action);
						actionsBySignature.put(action.getActionSignature(), action);
					}
				}
			}
		}
		
		return new AttilaActionLookup(actions, actionsBySignature);
	}
	
	/**
	 * Create a single <code>AttilaAction</code>.
	 * @param containerClass - a <code>Class</code> that has <code>HTTPMethod</code> annotated public methods
	 * @param methodName - the Java method that should be wrapped as an <code>AttilaAction</code>
	 * @param authenticateByDefault - true/false, depending whether the actions should be authenticated by default or not. If 
	 * method is not explicitly annotated with <code>Authentication</code>-annotation, this value is used to decide whether
	 * Attila should authenticate the action or not.
	 * */
	@SuppressWarnings("unchecked")
	public static AttilaAction createAction(Class containerClass, String methodName){
		Method m = searchMethod(methodName, containerClass.getMethods());
		return createAction(containerClass, m);
	}
	
	/**
	 * Create a single <code>AttilaAction</code>.
	 * @param containerClass - a <code>Class</code> that has <code>HTTPMethod</code> annotated public methods
	 * @param method - the Java method that should be wrapped as an <code>AttilaAction</code>
	 * @param authenticateByDefault - true/false, depending whether the actions should be authenticated by default or not. If 
	 * method is not explicitly annotated with <code>Authentication</code>-annotation, this value is used to decide whether
	 * Attila should authenticate the action or not.
	 * */
	@SuppressWarnings("unchecked")
	public static AttilaAction createAction(Class containerClass, Method method){
		boolean authenticateMethod = getControllerAuthenticationValue(containerClass);
		Authentication methodAuth = method.getAnnotation(Authentication.class);
		if(methodAuth != null){
			authenticateMethod = methodAuth.checkAuthentication();
		}
		
		AttilaAction action;
		HTTPMethod httpMethod = method.getAnnotation(HTTPMethod.class);
		if(httpMethod != null){
			action = new AttilaAction(method, authenticateMethod, httpMethod.method());					
		}else{			
			action = new AttilaAction(method, authenticateMethod);
		}
		
		if(authenticateMethod){
			AuthLevel authLevel = getAuthLevel(action, containerClass);
			if(authLevel != null){
				configureAuthenticationLevel(action, authLevel);				
			}
		}
		
		return action;
	}
	
	/**
	 * Returns AuthLevel of the given action or the AuthLevel of the controller
	 * if AuthLevel has not been defined separately for the action.
	 * */
	@SuppressWarnings("unchecked")
	private static AuthLevel getAuthLevel(AttilaAction action, Class controllerClass){
		AuthLevel methodAuthenticationLevel = action.getAction().getAnnotation(AuthLevel.class);
		if(methodAuthenticationLevel == null){
			AnnotatedElement classAnnotation = controllerClass;
			methodAuthenticationLevel = classAnnotation.getAnnotation(AuthLevel.class);
		}
		
		return methodAuthenticationLevel;
	}
	
	/**
	 * Set and configure AuthenticationLevel for the given AttilaAction.
	 * @param action - action to configure
	 * @param authenticationLevel - AuthLevel to configure with
	 * */
	@SuppressWarnings("unchecked")
	private static void configureAuthenticationLevel(AttilaAction action, AuthLevel authenticationLevel){
		action.setAuthenticationLevel(AuthenticationLevelFactory.createAuthenticationLevel(authenticationLevel));
		
		Class[] errorController = authenticationLevel.errorController();
		if(errorController.length == 1){
			action.setAuthenticationLevelErrorController(errorController[0]);
		}
		
		action.setAuthenticationLevelErrorControllerAction(authenticationLevel.errorControllerAction());
	}
	
	/**
	 * Parse default authentication setting from controller's annotations
	 * */
	@SuppressWarnings("unchecked")
	private static boolean getControllerAuthenticationValue(Class controllerClass){
		boolean authenticateControllerMethodsByDefault = false;
		AnnotatedElement classAnnotation = controllerClass;
		Authentication auth = classAnnotation.getAnnotation(Authentication.class);
		if(auth != null){
			authenticateControllerMethodsByDefault = auth.checkAuthentication();
		}
		
		return authenticateControllerMethodsByDefault;
	}
	
	/**
	 * Search for a method with the given name from and array of methods.
	 * @param methodName - name of the method
	 * @param methods - array of methods
	 * */
	private static Method searchMethod(String methodName, Method[] methods){
		for(Method m : methods){
			if(m.getName().equals(methodName)){
				return m;
			}
		}
		return null;
	}
	
}
