/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core.util;

import java.util.*;

import javax.servlet.http.*;

/**
 * Takes an url and parses an action and other useful data out of it.
 * @author Kai Kousa
 * */
public class RESTUrl {
	
	private String action = null;
	private String controller = null;
	private List<String> parameterValues = new ArrayList<String>();
	private String requestedUrl = "";
	
	private boolean index = false;
	private boolean root = false;
	
	/**
	 * Creates an instance of <code>RESTUrl</code>
	 * @param url - eg. http://example.com/testversion/users/user/24, where "http://example.com/testversion" is the context, 
	 * "users" is controller, "user" is method/action and "24" is a parameter
	 * @param appContext - context of the application, eg. http://example.com/testversion 
	 * */
	public RESTUrl(String url, String appContext){
		setRequestedUrl(url);
		parseUrl(url, appContext);
	}
	
	public RESTUrl(HttpServletRequest request){
		String requestUrl = request.getRequestURL().toString();
		String port = request.getServerPort() == 80 ? "" : ":" + request.getServerPort();
		String appContext = "http://" + request.getServerName() + port + request.getContextPath();
		
		setRequestedUrl(requestUrl);
		
		parseUrl(requestUrl, appContext);
	}
	
	public String getAction(){
		return action;
	}
	
	public String getController(){
		return controller;
	}
	
	public List<String> getParameterValues(){
		return parameterValues;
	}
	
	private void setRequestedUrl(String requestedUrl) {
		this.requestedUrl = requestedUrl;
	}

	public String getRequestedUrl() {
		return requestedUrl;
	}

	/**
	 * Returns true if no action has been found and the default index-action should be invoked
	 * */
	public boolean isIndex(){
		return index;
	}
	
	/**
	 * Returns true if the url is the same as the application context
	 * */
	public boolean isRoot(){
		return root;
	}
	
	public String getActionSignature(){
		return getAction() + ":" + getParameterValues().size();
	}
	
	/**
	 * Extracts controller, action and parameter sections from the url and stores them to member fields.
	 * */
	private void parseUrl(String url, String appContext){
		String tempUrl = url.replace(appContext, "").trim();
		
		if(tempUrl.length() > 0){
			if(tempUrl.startsWith("/")){
				tempUrl = tempUrl.replaceFirst("/", "");
			}
			
			String[] splitUrl = tempUrl.split("/");
			if(splitUrl.length == 1){//http://example.com/testversion/users
				controller = splitUrl[0];
				index = true;
			}else if(splitUrl.length == 2){//http://example.com/testversion/users/user
				controller = splitUrl[0];
				action = splitUrl[1];
			}else if(splitUrl.length > 2){//http://example.com/testversion/users/user/24
				controller = splitUrl[0];
				action = splitUrl[1];
				
				//extract parameters
				for(int i = 2; i < splitUrl.length; i++){
					parameterValues.add(splitUrl[i]);
				}			
			}			
		}else{
			index = true;
			root = true;
		}
		
	}
	
}
