/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.form;

import java.util.*;

import javax.servlet.http.*;

import attila.form.validators.*;

/**
 * Public interface for different form implementations. Each form is able 
 * to add fields and also link those fields together.
 * @author Kai Kousa
 * */
public interface Form {

	/**
	 * Adds a field to this form.
	 * @param name - name of the field
	 * @param validator - validation rule for this field
	 * @param errorText - error text that the UI might use if the field fails validation
	 * */
	public void addField(String name, Rule validator, String errorText);
	
	/**
	 * Adds a field to this form.
	 * @param name - name of the field
	 * @param validator - validation rule for this field
	 * @param errorText - error text that the UI might use if the field fails validation
	 * @param optional - true if the field is optional
	 * */
	public void addField(String name, Rule validator, String errorText, boolean optional);
	
	/**
	 * Adds a field to this form.
	 * @param field - an instance of {@link Field}
	 * */
	public void addField(Field field);
	
	/**
	 * Searches for a field.
	 * @param name - name of the field
	 * @return a field, or null if not found
	 * */
	public Field getField(String name);
	
	/**
	 * @return all fields contained in this Form.
	 * */
	public List<Field> getFields();
	
	/**
	 * Links two fields together and assigns a rule that validates the link.
	 * @param field1 - name of the first field
	 * @param field2 - name of the second field
	 * @param link - validation for the link (if the two fields should be equal, assign a FieldLink-implementation 
	 * that checks the equality of the fields)
	 * */
	public void linkFields(String field1, String field2, FieldLink link);
	

	/** Links two fields together and assigns a rule that validates the link.
	 * @param field1 - first field to be linked with the second field
	 * @param field2 - second field to be linked with the first field
	 * @param link - validation for the link (if the two fields should be equal, assign a FieldLink-implementation 
	 * that checks the equality of the fields)
	 * */
	public void linkFields(Field field1, Field field2, FieldLink link);
	
	/**
	 * @return a List of error messages gathered from the fields. If no errors have occured, 
	 * should return an empty list(and not null)
	 * */
	public List<String> getErrorMessages();
	
	/**
	 * Validates the fields in the form against the parameters found from the request.
	 * @param request - an instance of HttpServletRequest
	 * @return true if the fields validated without error, false otherwise
	 * */
	public boolean validate(HttpServletRequest request);
	
}
