/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.form;

import java.util.*;

import javax.servlet.http.HttpServletRequest;

import attila.form.validators.*;

/**
 * An implementation of {@link Form} with some additions. Each member field of type <code>Field</code> of the 
 * extending class will be automatically added to the list of validating fields. Fields added explicitly 
 * through various addField()-methods override all automatically added fields.
 * @author Kai Kousa
 * */
public class SimpleForm implements Form {

	protected List<Field> fields = new ArrayList<Field>();
	protected List<FieldLink> links = new ArrayList<FieldLink>();
	protected List<String> errorMessages = new ArrayList<String>();
	
	@Override
	public void addField(String name, Rule validator, String errorText) {
		addField(new SimpleField(name, validator, errorText));
	}
	
	@Override
	public void addField(String name, Rule validator, String errorText, boolean optional) {
		addField(new SimpleField(name, validator, errorText, optional));
	}
	
	public void addField(Field field){
		this.fields.add(field);
	}

	public List<String> getErrorMessages(){
		return errorMessages;
	}
	
	@Override
	public void linkFields(String field1, String field2, FieldLink link) {
		Field f1 = getField(field1);
		Field f2 = getField(field2);
		linkFields(f1, f2, link);
	}
	
	@Override
	public void linkFields(Field field1, Field field2, FieldLink link) {
		if(field1 != null && field2 != null){
			link.setLinkedFields(field1, field2);
			links.add(link);			
		}
	}
	
	@Override
	public boolean validate(HttpServletRequest request) {
		addMemberFields();
		boolean validationResult = true;

		//validate each field
		for(Field field : fields){
			String fieldValue = request.getParameter(field.getName());
			field.setValue(fieldValue);
			
			if(field.isOptional()){//Handle optional fields
				
				//The validation of optional fields is taken into account only
				//if the field contains a value
				if(!field.isEmpty()){
					if(!field.validate()){
						validationResult = false;
						errorMessages.add(field.getErrorText());
					}
				}
				
			}else{//Handle mandatory fields
				if(!field.validate()){
					validationResult = false;
					errorMessages.add(field.getErrorText());
				}								
			}
		}
		
		//validate each link
		for(FieldLink link : links){
			if(!link.validate()){
				validationResult = false;
				errorMessages.add(link.getErrorText());
			}
		}
		
		return validationResult;
	}
	
	/**
	 * Searches for a field in this form.
	 * */
	public Field getField(String name){
		for(Field field : fields){
			if(field.getName().equals(name)){
				return field;
			}
		}
		return null;
	}
	
	public List<Field> getFields(){
		return fields;
	}
	
	/**
	 * Looks for member fields in the class that are of type <code>Field</code>. Every instance of 
	 * Field that is found will be added to the form's list of fields if it has not already been added.
	 * */
	protected void addMemberFields(){
		java.lang.reflect.Field[] fields = this.getClass().getDeclaredFields();
		for(java.lang.reflect.Field field : fields){
			try {
				Object fieldValue = field.get(this);
				if(fieldValue instanceof Field){//Check the type
					Field formField = (Field) fieldValue;
					if(getField(formField.getName()) == null){//Check that the field doesn't exist in the list
						addField(formField);
					}
				}
			} catch (IllegalArgumentException e) {} 
			catch (IllegalAccessException e) {}
		}
	}
	
}
