/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.helper;

import java.sql.Timestamp;
import java.text.DateFormatSymbols;
import java.util.*;

/**
 * Useful helper for handling dates.
 * @author Kai Kousa
 * */
public class DateHelper {
	
	/**
	 * Returns today's date as an integer
	 * eg.: 06-06-2008 => 20080606
	 * */
	public static int getDateInt(){
		return getDateInt(0);
	}
	
	/**
	 * Returns chosen date as an integer
	 * @param advance - how many days to advance from current date (+/-)
	 * @return Date formatted as an integer
	 * */
	public static int getDateInt(int advance){
		Calendar cal = new GregorianCalendar();
		cal.add(Calendar.DAY_OF_MONTH, advance);

		return getDateInt(cal);
	}
	
	public static int getDateInt(int advance, int timezone){
		Calendar cal = new GregorianCalendar();
		cal.add(Calendar.DAY_OF_MONTH, advance);
		cal.add(Calendar.HOUR_OF_DAY, -timezone);
		
		return getDateInt(cal);
	}
	
	/**
	 * Returns the chosen date as an integer.
	 * @param cal - a <code>Calendar</code> that is set to the chosen date
	 * @return The chosen date formatted as an integer
	 * */
	public static int getDateInt(Calendar cal){
		int day = cal.get(Calendar.DAY_OF_MONTH);
		String dayString = addPreZero(day);
		
		int month = cal.get(Calendar.MONTH) + 1;
		String monthString = addPreZero(month);
		
		String date = "" + cal.get(Calendar.YEAR) + monthString + dayString;
		return Integer.parseInt(date);
	}
	
	/**
	 * Disassembles an integer formatted date and returns the previous or next day depending on the parameters.
	 * @param day - integer formatted date
	 * @param advance - 1 = next day, -1 = previous day. 
	 * */
	public static int getAdjacent(int day, int advance){
		Calendar cal = dateIntToCalendar(day);
		cal.add(Calendar.DAY_OF_MONTH, advance);
		return getDateInt(cal);
	}
	
	/**
	 * Converts an integer formatted date to an instance of <code>Calendar</code>
	 * @param dateInt - integer formatted date
	 * @return An instance of <code>Calendar</code> set to the date contained in dateInt.
	 * */
	public static Calendar dateIntToCalendar(int dateInt){
		return dateIntToCalendar(String.valueOf(dateInt));
	}
	
	/**
	 * Converts an integer formatted date to an instance of <code>Calendar</code>
	 * @param dateInt - integer formatted date as a String
	 * @return An instance of <code>Calendar</code> set to the date contained in dateInt.
	 * */
	public static Calendar dateIntToCalendar(String dateInt){
		if(dateInt.length() == 8){
			String yearString = "" + dateInt.charAt(0) + dateInt.charAt(1) + dateInt.charAt(2) + dateInt.charAt(3);
			String monthString = "" + dateInt.charAt(4) + dateInt.charAt(5);
			String dayString = "" + dateInt.charAt(6) + dateInt.charAt(7);
			
			Calendar cal = new GregorianCalendar();
			cal.set(Calendar.YEAR, Integer.parseInt(yearString));
			cal.set(Calendar.MONTH, Integer.parseInt(monthString)-1);//Months in java.util.Calendar start from zero(0) 
			cal.set(Calendar.DAY_OF_MONTH, Integer.parseInt(dayString));
			
			return cal;			
		}else{
			return null;
		}
	}
	
	/**
	 * Converts Timestamps to Calendars.
	 * @param time - Timestamp to be converted
	 * */
	public static Calendar timestampToCalendar(Timestamp time){
		
		Calendar cal = new GregorianCalendar();
		cal.setTime(time);
		
		return cal;
	}
	
	/**
	 * Converts an integer formatted to an instance of <code>Timestamp</code>
	 * @param dateInt - integer formatted date
	 * @return An instance of <code>Timestamp</code> set to the date contained in dateInt-parameter
	 * */
	public static Timestamp dateIntToTimestamp(int dateInt){
		Calendar calendar = dateIntToCalendar(dateInt);
		return new Timestamp(calendar.getTimeInMillis());
	}
	
	/**
	 * Converts timestamp to <code>Timestamp</code>-object
	 * @param timestampInt - timestamp in integer formatted form
	 * @return <code>Timestamp</code> or null if input is invalid
	 * */
	public static Timestamp timestampIntToTimestamp(long timestampInt){
		return timestampIntToTimestamp(String.valueOf(timestampInt));
	}
	
	/**
	 * Converts timestamp to <code>Timestamp</code>-object
	 * @param timestampInt - timestamp in integer formatted form
	 * @return <code>Timestamp</code> or null if input is invalid
	 * */
	public static Timestamp timestampIntToTimestamp(String timestampInt){
		if(timestampInt.length() == 12){
			String date = timestampInt.substring(0, 8);
			String time = timestampInt.substring(8, 12);
			
			String hourString = "" + time.charAt(0) + time.charAt(1);
			String minuteString = "" + time.charAt(2) + time.charAt(3);
			
			Calendar calendar = dateIntToCalendar(date);
			calendar.set(Calendar.HOUR_OF_DAY, Integer.parseInt(hourString));
			calendar.set(Calendar.MINUTE, Integer.parseInt(minuteString));

			return new Timestamp(calendar.getTimeInMillis());
		}else{
			return null;			
		}
	}
	
	public static long getLongTime(Calendar cal){
		int date = getDateInt(cal);
		int hour = cal.get(Calendar.HOUR_OF_DAY);
		int minute = cal.get(Calendar.MINUTE);
		
		String hourStr = addPreZero(hour);
		String minuteStr = addPreZero(minute);
		
		String time = date + hourStr + minuteStr;
		
		return Long.parseLong(time);		
	}
	
	/**
	 * Returns the short name of the weekday.
	 * @param dayOfWeek - values from 1 to 7 accepted (1 =Sunday, 7 = Saturday, 0 = empty)
	 * */
	public static String getShortWeekday(int dayOfWeek){
		return getShortWeekday(dayOfWeek, Locale.getDefault());
	}
	
	/**
	 * Returns the short name of the weekday.
	 * @param dayOfWeek - values from 1 to 7 accepted (1 =Sunday, 7 = Saturday, 0 = empty)
	 * */
	public static String getShortWeekday(int dayOfWeek, Locale locale){
		DateFormatSymbols symbols = new DateFormatSymbols(locale);
		String[] shorts = symbols.getShortWeekdays();

		String today = shorts[dayOfWeek];
		today = today.substring(0, 1).toUpperCase() + today.substring(1, today.length()).toLowerCase(); //First letter upper case, others lower case
		return today;
	}
	
	/**
	 * Return the long name of the weekday
	 * @param dayOfWeek - values from 1 to 7 accepted (1 = Sunday, 7 = Saturday, 0 = [empty])
	 * */
	public static String getLongWeekday(int dayOfWeek){
		return getLongWeekday(dayOfWeek, Locale.getDefault());
	}
	
	/**
	 * Return the long name of the weekday
	 * @param dayOfWeek - values from 1 to 7 accepted (1 = Sunday, 7 = Saturday, 0 = [empty])
	 * @param locale - 
	 * */
	public static String getLongWeekday(int dayOfWeek, Locale locale){
		DateFormatSymbols symbols = new DateFormatSymbols(locale);
		String[] longs = symbols.getWeekdays();
		
		String today = longs[dayOfWeek];
		today = today.substring(0, 1).toUpperCase() + today.substring(1, today.length()).toLowerCase(); //First letter upper case, others lower case
		return today;
	}
	
	/**
	 * @return the amount of days between one date and second
	 * */
	public static long daysBetweenCalendars(Calendar date1, Calendar date2){
		long d1 = getDaysFromYearZero(date1);
		long d2 = getDaysFromYearZero(date2);
		
		return d2 - d1;
	}
	
	/**
	 * @return the amount of days that have passed since the year zero and the given date
	 * */
	public static long getDaysFromYearZero(Calendar date){
		return ((date.get(Calendar.YEAR) * 365) - 365) + date.get(Calendar.DAY_OF_YEAR);
	}
	
	/**
	 * Adds a zero (0) to the string if the given value is below ten (10).
	 * */
	private static String addPreZero(int value){
		if(value < 10)
			return "0" + value;
		else
			return "" + value;
	}
	
}
