/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.helper;

import java.util.*;

import javax.servlet.http.*;

import attila.form.validators.*;

/**
 * Contains helper methods for reading data from requests.
 * @author Kai Kousa
 * */
public class RequestHelper {

	/**
	 * Reads a <code>String</code>-parameter from request.
	 * @return null if the parameter value is corrupted
	 * */
	public static String readString(String param, HttpServletRequest request){
		Rule validationRule = new RegExpRule(){
			@Override
			public String getPattern() {
				return ".+";
			}						
		};
		return readString(param, request, validationRule);
	}
	
	/**
	 * Reads a <code>String</code>-parameter from request.
	 * @param param - the parameter that is read
	 * @param request - request where the value is read from
	 * @param rule - rule against which the parameter value is validated
	 * @return null if the parameter value is corrupted
	 * */
	public static String readString(String param, HttpServletRequest request, Rule rule){
		String s = getParam(param, request);
		
		if(s != null){
			if(rule.validate(s))
				return s;
			else
				return null;			
		}else{
			return null;
		}
	}
	
	/**
	 * Reads an <code>int</code>-parameter from request.
	 * @param param - the parameter that is read
	 * @param request - request where the value is read from
	 * @param rule - rule against which the parameter value is validated
	 * @return 0 if the parameter value is corrupted
	 * */
	public static int readInt(String param, HttpServletRequest request){
		return readInt(param, request, new IntRule());		
	}
	
	/**
	 * Reads an <code>int</code>-parameter from request.
	 * @return 0 if the parameter value is corrupted
	 * */
	public static int readInt(String param, HttpServletRequest request, Rule rule){
		String s = getParam(param, request);
		
		if(s != null){
			int i = 0;
			if(rule.validate(s)){
				try{
					i = Integer.parseInt(s);
				}catch(NumberFormatException numEx){
					i = 0;
				}
				
			}
			
			return i;			
		}else{
			return 0;
		}
	}
	
	/**
	 * Reads a <code>float</code>-parameter from request.
	 * @return 0 if the parameter value is corrupted
	 * */
	public static float readFloat(String param, HttpServletRequest request){
		String s = getParam(param, request);			
		
		if(s != null){
			float f = 0;
			if(new FloatRule().validate(s))
				f = Float.parseFloat(s);
			
			return f;			
		}else{
			return 0;
		}
		
	}
	
	public static long readLong(String param, HttpServletRequest request){
		String s = getParam(param, request);
		
		if(s != null){
			Rule rule = new IntRule();
			long l = 0;
			if(rule.validate(s))
				l = Long.parseLong(s);
			return l;
		}else{
			return 0;
		}
	}
	
	/**
	 * Reads boolean values from the request. 1 and 0 are interpreted as true and false respectively.
	 * */
	public static boolean readBoolean(String param, HttpServletRequest request){
		String s = getParam(param, request);		
		if(s.equals("true") || s.equals("1")){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Checks if a parameter has been sent.
	 * */
	public static boolean isSet(String param, HttpServletRequest request){
		String s = getParam(param, request);
		if(s != null){
			return true;
		}else{
			return false;
		}
	}
	
	/**
	 * Reads values for a parameter that can have multiple values
	 * @return <code>List&lt;String&gt;</code> containing the parameter values
	 * */
	public static List<String> readStringList(String param, HttpServletRequest request){
		return RequestHelper.readStringList(param, request, new ScandicTextRule());
	}
	
	public static List<String> readStringList(String param, HttpServletRequest request, Rule rule){
		List<String> stringList = new ArrayList<String>();
		
		String[] values = request.getParameterValues(param);
		if(values != null){
			for(String s : values){
				if(rule.validate(s))
					stringList.add(s);
			}
		}
		
		return stringList;
	}
	
	/**
	 * Reads values for a parameter that can have multiple values
	 * @return <code>List&lt;Integer&gt;</code> containing the parameter values
	 * */
	public static List<Integer> readIntList(String param, HttpServletRequest request){
		List<Integer> intList = new ArrayList<Integer>();
		
		String[] values = request.getParameterValues(param);
		if(values != null){
			Rule rule = new IntRule();
			for(String s : values){
				if(rule.validate(s))
					intList.add(Integer.parseInt(s));
			}			
		}
		
		return intList;
	}
	
	/**
	 * Reads values for a parameter that can have multiple values
	 * @return <code>List&lt;Float&gt;</code> containing the parameter values
	 * */
	public static List<Float> readFloatList(String param, HttpServletRequest request){
		List<Float> floatList = new ArrayList<Float>();
		
		String[] values = request.getParameterValues(param);
		if(values != null){
			Rule rule = new FloatRule();
			for(String s : values){
				if(rule.validate(s))
					floatList.add(Float.parseFloat(s));
			}
		}
		
		return floatList;
	}
	
	/**
	 * Determines whether a submit button has been pressed.
	 * @return true - if the parameter was found => the button was pressed
	 * */
	@SuppressWarnings("unchecked")
	public static boolean readCommand(String param, HttpServletRequest request){
		Map paramMap = request.getParameterMap();
		return paramMap.containsKey(param);
	}
	
	/**
	 * Creates a type safe map of parameters in the request
	 * */
	@SuppressWarnings("unchecked")
	public static Map<String, String> getParameterMap(HttpServletRequest request){
		Map<String, String> params = new HashMap<String, String>();
		
		Enumeration paramNames = request.getParameterNames();
		while(paramNames.hasMoreElements()){
			String paramName = (String) paramNames.nextElement();
			String paramValue = request.getParameter(paramName);
			
			params.put(paramName, paramValue);
		}
		
		return params;
	}
	
	/**
	 * Try reading value of the given parameter from the request's parameters. If it fails try 
	 * reading from request's attributes. 
	 * */
	private static String getParam(String param, HttpServletRequest request){
		String s = request.getParameter(param);
		return s;
	}
	
}
