/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.jsp.functions;

import java.text.*;
import java.util.*;

/**
 * Used for converting a Calendar to a formatted date/time String.
 * Conversions are handled by {@link java.text.SimpleDateFormat SimpleDateFormat}. Check
 * SimpleDateFormat's JavaDoc on how to use patterns.
 * @author Eric Andrews
 * */
public class CalendarFormatFunction {

	/**
	 * Formats a Calendar according to the given pattern using system defaults for locale and time zone.
	 * @param calendar	offers the date and time data.
	 * @param pattern	defines how the formatted string should be built.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String format(Calendar calendar, String pattern) {
		return formatWithLocaleAndTimeZone(calendar, pattern, Locale.getDefault(), TimeZone.getDefault());
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given locale and 
	 * system default time zone.
	 * @param calendar	offers the date and time data.
	 * @param pattern	defines how the formatted string should be built.
	 * @param locale	affects things like weekday names.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithLocale(Calendar calendar, String pattern, Locale locale) {
		return formatWithLocaleAndTimeZone(calendar, pattern, locale, TimeZone.getDefault());
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given locale id and 
	 * system default time zone.
	 * @param calendar	offers the date and time data.
	 * @param pattern	defines how the formatted string should be built.
	 * @param localeId	used to instantiate a {@link java.util.Locale Locale}.
	 * 					Affects things like weekday names.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithLocaleId(Calendar calendar, String pattern, String localeId) {
		return formatWithLocale(calendar, pattern, new Locale(localeId));
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given time zone and
	 * system defaults for locale.
	 * @param calendar	offers the date and time data.
	 * @param pattern	defines how the formatted string should be built.
	 * @param timeZone	affects the hour field of the result.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithTimeZone(Calendar calendar, String pattern, TimeZone timeZone) {
		return formatWithLocaleAndTimeZone(calendar, pattern, Locale.getDefault(), timeZone);
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given time zone id and
	 * system defaults for locale.
	 * @param calendar		offers the date and time data.
	 * @param pattern		defines how the formatted string should be built.
	 * @param timeZoneId	used to instantiate a {@link java.util.TimeZone TimeZone}.
	 * 						Affects the hour field of the result.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithTimeZoneId(Calendar calendar, String pattern, String timeZoneId) {
		return formatWithTimeZone(calendar, pattern, TimeZone.getTimeZone(timeZoneId));
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given locale and time zone.
	 * @param calendar	offers the date and time data.
	 * @param pattern	defines how the formatted string should be built.
	 * @param locale	affects things like weekday names.
	 * @param timeZone	affects the hour field of the result.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithLocaleAndTimeZone(Calendar calendar, String pattern,
			Locale locale, TimeZone timeZone) {
		if (calendar == null) {
			return null;
		} else {
			SimpleDateFormat df = new SimpleDateFormat(pattern);
			df.setDateFormatSymbols(new DateFormatSymbols(locale));
			df.setTimeZone(timeZone);
			return df.format(calendar.getTime());
		}
	}
	
	/**
	 * Formats a Calendar according to the given pattern, using the given locale id and time zone id.
	 * @param calendar		offers the date and time data.
	 * @param pattern		defines how the formatted string should be built.
	 * @param localeId		used to instantiate a {@link java.util.Locale Locale}.
	 * 						Affects things like weekday names.
	 * @param timeZoneId	used to instantiate a {@link java.util.TimeZone TimeZone}.
	 * 						Affects the hour field of the result.
	 * @return	a formatted string representation of the calendar.
	 */
	public static String formatWithLocaleAndTimeZoneIds(Calendar calendar, String pattern,
			String localeId, String timeZoneId) {
		return formatWithLocaleAndTimeZone(calendar, pattern,
				new Locale(localeId), TimeZone.getTimeZone(timeZoneId));
	}
	
}
