/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import junit.framework.*;

import attila.core.util.*;
import attila.mock.*;

/**
 * Test cases for <code>RESTActionInvoker</code>
 * @author Kai Kousa
 * */
public class RESTActionInvokerTests extends TestCase {

	private HttpServletRequestMock mockRequest = new HttpServletRequestMock(); 
	private HttpServletResponseMock mockResponse = new HttpServletResponseMock();
	
	/**
	 * Tests support for automatic casting to integers
	 * */
	public void testInvokeAction_intParameter(){
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/12"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "bar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Tests support for automatic casting to Strings (or non-casting actually)
	 * */
	public void testInvokeAction_stringParameter(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/Kai"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "stringbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Tests automatic casting to Floats
	 * */
	public void testInvokeAction_floatParameter(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/2.1"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "floatbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	public void testInvokeAction_longParameter(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/" + Long.MAX_VALUE));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "longbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Test handling of boolean values
	 * */
	public void testInvokeAction_booleanParameter(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/true"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "booleanbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Test handling of boolean values if the boolean is presented as 1 or 0
	 * */
	public void testInvokeAction_numericBooleanValue(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/0"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "booleanbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Test handling of boolean values if the action's parameter is defined 
	 * as a non-primitive Boolean object.
	 * */
	public void testInvokeAction_objectBooleaParameter(){
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/false"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionInvoker invoker = new RESTActionInvoker();
		AttilaAction action = ActionParser.createAction(FooController.class, "objectbooleanbar");
		FooController controller = new FooController();
		try{
			invoker.invokeAction(action, controller, mockRequest, mockResponse);
			assertTrue(true);
		}catch(Exception ex){
			ex.printStackTrace();
			fail();
		}
	}
	
	/**
	 * Dummy controller for testing.
	 * */
	@SuppressWarnings({ "unused", "serial" })
	private class FooController extends AttilaTheController{

		@Override
		public void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {
			
		}
		
		@HTTPMethod
		public void bar(int i, HttpServletRequest request, HttpServletResponse response){
			System.out.println("Integer is " + i);
		}
		
		public void stringbar(String s, HttpServletRequest request, HttpServletResponse response){
			System.out.println("String is " + s);
		}
		
		public void floatbar(float f, HttpServletRequest request, HttpServletResponse response){
			System.out.println("Float is " + f);
		}
		
		public void longbar(long l, HttpServletRequest request, HttpServletResponse response){
			System.out.println("Long is " + l);
		}
		
		public void booleanbar(boolean b, HttpServletRequest request, HttpServletResponse response){
			System.out.println("b is " + b);
		}
		
		public void objectbooleanbar(Boolean ob, HttpServletRequest request, HttpServletResponse response){
			System.out.println("ob is " + ob);
		}
		
	}
	
}
