/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import junit.framework.*;

import attila.mock.*;
import attila.core.util.*;

/**
 * Test cases for <code>RESTActionResolver</code>-class.
 * @author Kai Kousa
 * */
public class RESTActionResolverTests extends TestCase{

	/**
	 * Given a request to a specified url, the action resolver must be able to find an action from 
	 * our dummy controller.
	 * */
	public void testResolveAction(){
		AttilaActionLookup lookup = ActionParser.createActionLookup(FooController.class, HttpMethodEnum.GET);
		
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/12"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionResolver resolver = new RESTActionResolver();
		AttilaAction action = resolver.resolveAction(new AttilaRequest(mockRequest), lookup);
		
		assertEquals("bar", action.getName());
	}
	
	public void testResolveAction_portNumberInUrl(){
		AttilaActionLookup lookup = ActionParser.createActionLookup(FooController.class, HttpMethodEnum.GET);
		
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com:8080/foo/bar/12"));
		mockRequest.setServerName("example.com");
		mockRequest.setServerPort(8080);
		mockRequest.setContextPath("/");
		
		ActionResolver resolver = new RESTActionResolver();
		AttilaAction action = resolver.resolveAction(new AttilaRequest(mockRequest), lookup);
		
		assertEquals("bar", action.getName());
	}
	
	/**
	 * Check that the resolver is able to resolve actions with the same name but different number of parameters.
	 * */
	public void testResolveAction_overloadedActions(){
		AttilaActionLookup lookup = ActionParser.createActionLookup(OverloadedController.class, HttpMethodEnum.GET);
		
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar"));
		mockRequest.setServerName("example.com");
		mockRequest.setContextPath("/");
		
		ActionResolver resolver = new RESTActionResolver();
		AttilaAction action = resolver.resolveAction(new AttilaRequest(mockRequest), lookup);
		
		assertEquals(2, action.getAction().getParameterTypes().length);
		
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/1"));
		action = resolver.resolveAction(new AttilaRequest(mockRequest), lookup);
		
		assertEquals(3, action.getAction().getParameterTypes().length);
		
		mockRequest.setRequestURL(new StringBuffer("http://example.com/foo/bar/1/2"));
		action = resolver.resolveAction(new AttilaRequest(mockRequest), lookup);
		
		assertEquals(4, action.getAction().getParameterTypes().length);
	}
	
	@SuppressWarnings({ "unused", "serial" })
	private class FooController extends AttilaTheController{
		
		@Override
		public void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {}
		
		@HTTPMethod
		public void bar(int a, HttpServletRequest request, HttpServletResponse response){}
		
	}
	
	/**
	 * An example of a controller with multiple overloaded actions.
	 * */
	@SuppressWarnings({ "unused", "serial" })
	private class OverloadedController extends AttilaTheController{

		@Override
		public void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {}
		
		@HTTPMethod
		public void bar(HttpServletRequest request, HttpServletResponse response){}
		
		@HTTPMethod
		public void bar(int a, HttpServletRequest request, HttpServletResponse response){}
		
		@HTTPMethod
		public void bar(int a, int b, HttpServletRequest request, HttpServletResponse response){}
		
	}
	
}
