/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.core.util;

import java.io.IOException;

import javax.servlet.ServletException;
import javax.servlet.http.*;

import junit.framework.*;

import attila.core.*;
import attila.core.auth.*;

/**
 * Test cases for <code>ActionParser</code>
 * @author Kai Kousa
 * */
public class ActionParserTests extends TestCase {

	/**
	 * Check that the parser returns 2 actions that have been annotated as GET-methods
	 * */
	public void testCreateActionLookup_GET(){		
		AttilaActionLookup actions = ActionParser.createActionLookup(ActionClass.class, HttpMethodEnum.GET);
		
		assertEquals(2, actions.getLookupMap().size());
	}
	
	/**
	 * Check that the parser returns the one action that has been annotated as POST-method
	 * */
	public void testCreateActionLookup_POST(){		
		AttilaActionLookup actions = ActionParser.createActionLookup(ActionClass.class, HttpMethodEnum.POST);
		
		assertEquals(1, actions.getLookupMap().size());
	}
	
	/**
	 * Test that the parser returns the 3 actions that have been annotated in both the subclass and the superclass.
	 * */
	public void testSubClass(){
		ActionSubClass subClass = new ActionSubClass();
		
		AttilaActionLookup actions = subClass.getSubClassActions();
		
		assertEquals(3, actions.getLookupMap().size());
		assertTrue(actions.containsAction("doSomethingSomething"));
	}
	
	/**
	 * Test that the parser handles servlets as well. Meaning that all the non-annotated methods are filtered out.
	 * */
	public void testParseServlet(){
		AttilaActionLookup actions = ActionParser.createActionLookup(ServletDummy.class, HttpMethodEnum.GET);
		assertEquals(1, actions.getLookupMap().size());
	}
	
	public void testCreateAction(){
		AttilaAction action = ActionParser.createAction(ServletDummy.class, "index");
		
		assertEquals("index", action.getAction().getName());
	}
	
	public void testParseAuthenticationLevel(){
		AttilaAction action = ActionParser.createAction(AuthenticatedController.class, "dosomethingauthenticated");
		assertEquals("attila.core.util.FooAuthLevel", action.getAuthenticationLevel().getClass().getName());
	}
	
	/**
	 * A simple class for testing parsing.
	 * */
	@SuppressWarnings("unused")
	private class ActionClass{
		
		@HTTPMethod
		public void doSomething(){}
		
		@HTTPMethod
		public void doSomethingAnother(){}
		
		@HTTPMethod(method=HttpMethodEnum.POST)
		public void doSomethingWithPOST(){}
		
		public AttilaActionLookup getSubClassActions(){
			return ActionParser.createActionLookup(this.getClass(), HttpMethodEnum.GET);
		}
		
	}
	
	/**
	 * A simple class to test how subclasses are handled by the parser.
	 * */
	@SuppressWarnings("unused")
	private class ActionSubClass extends ActionClass{
		
		@Authentication(checkAuthentication=false)
		@HTTPMethod
		public void doSomethingSomething(){}
		
	}
	
	/**
	 * A simple subclass of AttilaTheController to check that the parser handles the 'real' cases also.
	 * */
	private class ServletDummy extends AttilaTheController{
		static final long serialVersionUID = 2792387283001479564L;

		@Override
		public void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {}
		
		@SuppressWarnings("unused")
		@HTTPMethod
		public void something(HttpServletRequest request, HttpServletResponse response){}
		
	}
	
	@SuppressWarnings("serial")
	private class AuthenticatedController extends AttilaTheController{

		@Override
		public void index(HttpServletRequest request, HttpServletResponse response) throws ServletException, IOException {}
		
		@Authentication
		@AuthLevel(level=FooAuthLevel.class)
		@SuppressWarnings("unused")
		public void dosomethingauthenticated(HttpServletRequest request, HttpServletResponse response){}
		
	}
	
}
