/*
* Copyright 2009-2011 Telkku.com
*
* Licensed under the Apache License, Version 2.0 (the "License");
* you may not use this file except in compliance with the License.
* You may obtain a copy of the License at
*
* http://www.apache.org/licenses/LICENSE-2.0
*
* Unless required by applicable law or agreed to in writing, software
* distributed under the License is distributed on an "AS IS" BASIS,
* WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
* See the License for the specific language governing permissions and
* limitations under the License.
*/

package attila.form;

import java.util.*;

import junit.framework.*;

import attila.mock.*;
import attila.form.validators.*;

/**
 * Test cases for the form-framework.
 * @author Kai Kousa
 * */
public class FormTests extends TestCase{

	private Rule usernameRule = new EmptyRule();
	private Rule passwordRule = new EmptyRule();
	
	private Rule passwordRegExpRule = new RegExpRule("[a-zA-Z0-9]+");
	
	@Override
	protected void setUp() throws Exception {
		super.setUp();
		usernameRule = usernameRule.add(new NotNullRule()).add(new StringMinLengthRule(5));
		
		passwordRule = passwordRule.add(new NotNullRule()).add(new StringMinLengthRule(5));
	}
	
	/**
	 * Validate input that contains parameters in correct format.
	 * Validation should succeed.
	 * */
	public void testValidate_correctInput(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("username", "kaitsu");
		parameters.put("password", "salasana2342");
		
		HttpServletRequestMock mock = new HttpServletRequestMock();
		mock.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("username", usernameRule, "illegal username");
		form.addField("password", passwordRegExpRule, "illegal password");
		
		assertTrue(form.validate(mock));
	}
	
	/**
	 * Validate input that contains parameters in the WRONG format. Validation 
	 * should fail.
	 * */
	public void testValidate_incorrectInput(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("username", "kai"); //too short, so the validation should fail
		parameters.put("password", "2tf343");
		
		HttpServletRequestMock mock = new HttpServletRequestMock();
		mock.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("username", usernameRule, "illegal username");
		form.addField("password", passwordRegExpRule, "illegal password");
		
		assertFalse(form.validate(mock));
		assertEquals("illegal username", form.getErrorMessages().get(0));
	}
	
	/**
	 * Test linking of the two fields that are not equal. The form should notice this
	 * and fail the validation.
	 * */
	public void testLinkedFields_notEqual(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("password1", "pass123");
		parameters.put("password2", "paSS987");
		
		HttpServletRequestMock mock = new HttpServletRequestMock();
		mock.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("password1", passwordRegExpRule, "password incorrect");
		form.addField("password2", passwordRegExpRule, "password incorrect");
		
		FieldLink equalFields = new EqualFieldsLink();
		equalFields.setErrorText("fields are not equal");
		form.linkFields("password1", "password2", equalFields);
		
		assertFalse(form.validate(mock));
		
		assertEquals("fields are not equal", form.getErrorMessages().get(0));
	}
	
	/**
	 * Test linking of two fields that contain equal values. Validation 
	 * should succeed.
	 * */
	public void testLinkedFields_equalFields(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("password1", "pass123");
		parameters.put("password2", "pass123");
		
		HttpServletRequestMock mock = new HttpServletRequestMock();
		mock.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("password1", passwordRegExpRule, "password incorrect");
		form.addField("password2", passwordRegExpRule, "password incorrect");
		
		FieldLink equalFields = new EqualFieldsLink();
		equalFields.setErrorText("fields are not equal");
		
		form.linkFields("password1", "password2", equalFields);
		
		assertTrue(form.validate(mock));
	}
	
	/**
	 * Optional fields are validated only if they contain a value. In this case 
	 * email-field is left null, so the form should skip validating it and 
	 * the validation should succeed.
	 * */
	public void testOptionalFields(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("username", "kai");
		parameters.put("password", "kaitsu");
		parameters.put("email", null);
		
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("username", new ScandicTextRule(), "username incorrect");
		form.addField("password", passwordRegExpRule, "password incorrect");
		form.addField("email", new EmailRule(), "email incorrect", true);
		
		assertTrue(form.validate(mockRequest));
	}
	
	/**
	 * Here the optional email-field contains an invalid value. The form
	 * should try validating the value and fail the validation process. 
	 * */
	public void testOptionalFieldFail(){
		Map<String, String> parameters = new HashMap<String, String>();
		parameters.put("username", "kai");
		parameters.put("password", "kaitsu");
		parameters.put("email", "muu.com");
		
		HttpServletRequestMock mockRequest = new HttpServletRequestMock();
		mockRequest.setParameters(parameters);
		
		Form form = new SimpleForm();
		form.addField("username", new ScandicTextRule(), "username incorrect");
		form.addField("password", passwordRegExpRule, "password incorrect");
		form.addField("email", new EmailRule(), "email incorrect", true);
		
		assertFalse(form.validate(mockRequest));
	}
	
}
